#!perl -w -I \temp\1\bin
#
#######################################################################
package chunked;
use strict;
use IO::File;
use stkutils::ini_file;
use stkutils::chunked_file;
use Cwd;
sub new {
	my $class = shift;
	my $self = {};
	$self->{shaders} = {};
	$self->{shader_names} = {};
	bless($self, $class);
	return $self;
}
sub read {
	my $self = shift;
	my ($fn) = @_;

	my $i = 0;
	my $cf = stkutils::chunked_file->new($fn, 'r') or die;
	while (1) {
		my ($index, $size) = $cf->r_chunk_open();
		defined($index) or last;
		if ($index==0) {
			$self->read_section0($cf) if ($size > 0);
		} elsif ($index==1) {
			$self->read_section1($cf);
		} elsif ($index==2) {
			$self->read_shaders($cf);
		} elsif ($index==3) {
			$self->read_shader_names($cf);
		} else {
			die;
		}
		$cf->r_chunk_close();
	}
	$cf->close();
}
sub write {
	my $self = shift;
	my ($fn) = @_;
	my $cf = stkutils::chunked_file->new($fn, 'w') or die;
	my $i = 0;
	$cf->w_chunk_open(0);
	if (defined $self->{section0}) {
		$cf->w_chunk_data($self->{section0});
	}
	$cf->w_chunk_close();
	$cf->w_chunk_open(1);
	$cf->w_chunk_data($self->{section1});
	$cf->w_chunk_close();
	$cf->w_chunk_open(2);
	while ($i < $self->{shaders_count}) {
		$cf->w_chunk($i, $self->{shaders}->{$i});
		$i++;
	}
	$cf->w_chunk_close();
	$cf->w_chunk_open(3);
	$i = 0;
	$cf->w_chunk_data(pack('V', $self->{shaders_count}));
	while ($i < $self->{shaders_count}) {
		$cf->w_chunk_data(pack('Z*',$self->{shader_names}->{$i}));
		$i++;
	}	
	$cf->w_chunk_close();
	$cf->close();
}
sub import {
	my $self = shift;
	
	my $bin_fh = IO::File->new('section0.bin', 'r');
	if (defined $bin_fh) {
		binmode $bin_fh;
		$bin_fh->read($self->{section0}, ($bin_fh->stat())[7]);	
		$bin_fh->close();
	}

	$bin_fh = IO::File->new('section1.bin', 'r') or die "cannot open section1.bin\n";
	binmode $bin_fh;
	$bin_fh->read($self->{section1}, ($bin_fh->stat())[7]);
	$bin_fh->close();
	
	my $dir_handle;
	my $dir_handle_1;
	my $dir_handle_2;
	my $work_dir = getcwd();
	my $shader_root;
	my $shader_name;
	my $in_shader_name;
	opendir $dir_handle, $work_dir;
	my $i = 0;
	my $count = 0;
	while ($shader_root = readdir $dir_handle) {
		if ((-d $shader_root) and ($i > 1)) {
			my $new_dir = ''.$work_dir.'/'.$shader_root.'';
			opendir $dir_handle_1, $new_dir ;
			my $j = 0;
			while ($shader_name = readdir $dir_handle_1) {
				if ((-d $shader_root.'/'.$shader_name) and ($j > 1)) {
					my $new_dir = ''.$work_dir.'/'.$shader_root.'/'.$shader_name.'';
					opendir $dir_handle_2, $new_dir ;
					my $k = 0;
					while ($in_shader_name = readdir $dir_handle_2) {
						if ((-d $in_shader_name) and ($k > 1)) {
							print "there is some catalogs left";
						} elsif (($k > 1) and (substr($in_shader_name, -3) eq '.sh')) {
							my $fn = $shader_root.'/'.$shader_name.'/'.$in_shader_name;
							my $shader = IO::File->new($fn, 'r') or die "cannot open $fn\n";
							binmode $shader;
							$shader->read($self->{shaders}->{$count}, ($shader->stat())[7]);
							$self->{shader_names}->{$count} = substr($shader_root.'\\'.$shader_name.'\\'.$in_shader_name, 0, -3);
							$count++;
						} else {
							$k++;
						}
					}
					closedir $dir_handle_2;
				} elsif (($j > 1) and (substr($shader_name, -3) eq '.sh')) {
					my $fn = $shader_root.'/'.$shader_name;
					my $shader = IO::File->new($fn, 'r') or die "cannot open $fn\n";
					binmode $shader;
					$shader->read($self->{shaders}->{$count}, ($shader->stat())[7]);
					$self->{shader_names}->{$count} = substr($shader_root.'\\'.$shader_name, 0, -3);
					$count++;
				} else {
					$j++;
				}
			}
			closedir $dir_handle_1;
		} elsif (($i > 1) and (substr($shader_root, -3) eq '.sh')) {
			my $fn = $shader_root;
			my $shader = IO::File->new($fn, 'r') or die "cannot open $fn\n";
			binmode $shader;
			$shader->read($self->{shaders}->{$count}, ($shader->stat())[7]);
			$self->{shader_names}->{$count} = substr($fn, 0, -3);
			$count++;
		} else {
			$i++;
		}
	}
	closedir $dir_handle;
	$self->{shaders_count} = $count;
}
sub export {
	my $self = shift;
	
	if (defined $self->{section0}) {
		my $sect = IO::File->new('section0.bin', 'w') or die;
		binmode $sect;
		$sect->write($self->{section0}, length($self->{section0}));
		$sect->close();
	}
	
	my $sect = IO::File->new('section1.bin', 'w') or die;
	binmode $sect;
	$sect->write($self->{section1}, length($self->{section1}));
	$sect->close();
	
	my $work_dir = getcwd();
	my $i = 0;
	while ($i < $self->{shaders_count}) {
		my $string = $self->{shader_names}->{$i};
		my $out;
		my $shader_name;
		my $inc_folder;
		if ($string =~ /^(\w+)\\(\w.*)/) {
			$out = $1;
			$shader_name = $2;
			if ($shader_name =~ /^(\w+)\\(\w.*)/) {
				$inc_folder = $1;
				$shader_name = $2;
			}
		} else  {
			$shader_name = $string;
		}
		if (defined $out) {
			chdir $out or File::Path::mkpath($out, 0) or die "cannot make path $out\n";
			chdir ($out);
		}
		my $fh = ''.$shader_name.'.sh';
		if (defined $inc_folder) {
			File::Path::mkpath($inc_folder, 0);
			chdir $inc_folder;
		}
		my $bin_fh = IO::File->new($fh, 'w') or die "cannot open $fh\n";
		binmode $bin_fh;
		$bin_fh->write($self->{shaders}->{$i}, length($self->{shaders}->{$i}));
		$bin_fh->close();
		$i++;
		chdir $work_dir or die;
	}
}
sub read_shaders {
	my $self = shift;
	my ($cf) = @_;
	
	my $i = 0;
	while (1) {
		my ($index, $size) = $cf->r_chunk_open();
		defined $index or last;
		$self->{shaders}->{$i} = $cf->r_chunk_data();
		$cf->r_chunk_close();
		$i++;
	}
	$self->{shaders_count} = $i;
}
sub read_section0 {
	my $self = shift;
	my ($cf) = @_;
	$self->{section0} = $cf->r_chunk_data();
}
sub read_section1 {
	my $self = shift;
	my ($cf) = @_;
	$self->{section1} = $cf->r_chunk_data();
}
sub read_shader_names {
	my $self = shift;
	my ($fn) = @_;
	
	my $data = $fn->r_chunk_data();
	$self->{shader_names}->{count} = unpack ('V',$data);
	my $packet = stkutils::data_packet->new(substr($data, 4));
	my $i = 0;
	while ($i < $self->{shader_names}->{count}) {
		my @value = $packet->unpack('Z*');
		$self->{shader_names}->{$i} = $value[0];
		$i++;
	}
}
#######################################################################
package main;
use strict;
use Getopt::Long;
use File::Path;
use stkutils::data_packet;
use Cwd;

sub usage {
	return <<END
Chunk file splitter
Usage: cfs [-d input_file] [-o outdir]
       cfs [-c input_dir] [-o outfile]
END
}

my $input_file;
my $src_dir;
my $out;

GetOptions(
	'd=s' => \$input_file,
	'c=s' => \$src_dir,
	'o=s' => \$out,
) or die usage();

my $work_dir = getcwd();

if (defined $input_file) {
	die "bad params\n" if (defined $src_dir);
	my $file = chunked->new();
print "reading $input_file...\n";	
	$file->read($input_file);
	defined $out && do {
		File::Path::mkpath($out, 0);
		chdir $out or die "cannot change path to $out\n";
	};
print "exporting shaders...\n";	
	$file->export();	
	print "done!\n";
} else {
	die "bad params\n" if (defined $input_file);
	if (defined $src_dir) {
		chdir $src_dir or die "cannot change dir to $src_dir\n";
	}
	my $file = chunked->new();
	$out = 'chunked_file.new' unless defined $out;
print "importing shaders...\n";
	$file->import();
print "writing $out...\n";
	chdir $work_dir;
	$file->write($out);
print "done!\n";
}
#######################################################################